<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;
use App\Http\Controllers\SharedController;


class BOController extends Controller
{
    private $table_name;
    private $query;
    private $selections = array();
    private $group_by = array();
    private $is_form = false;

    private function getStruct(){
        $query = "SELECT DATA_TYPE as type, COLUMN_NAME as `key`, CASE WHEN COLUMN_KEY = 'PRI' THEN 0 ELSE CASE WHEN COLUMN_KEY = 'MUL' THEN 1 ELSE 2 END END as enabled, CASE WHEN IS_NULLABLE = 'NO' THEN 1 ELSE 0 END as required 
        FROM INFORMATION_SCHEMA.COLUMNS
        WHERE 
             TABLE_NAME = '".$this->table_name."' AND COLUMN_NAME <> 'created_date' AND COLUMN_NAME <> 'slug'";

        return DB::select(DB::raw($query));
    }

    public function structures($table_name){
        $this->table_name = 't_'.$table_name;
        return $this->getStruct();
    }

    private function getSize(){
        return DB::table($this->table_name)->count();
    }

    public function all($table_name,Request $request){
        $this->table_name = 't_'.$table_name;
        $limit = $request->input('limit');
        $offset = $request->input('offset');
        $order = $request->input('order') ? $request->input('order') : 'DESC';
        $by = ($request->input('by') ? $request->input('by') : 'id');
        $filter_key = $request->input('filter_key');
        $value = $request->input('filter_val');
        $joins = json_decode($request->input('joins'));

        $selections = $this->getStruct();        

        $this->query = DB::table($this->table_name);

        $size = $this->query->count();
       
        if(isset($joins)){
            $this->join($joins, $this->table_name);
            $this->query->select($this->table_name.'.*', DB::raw(implode(',', $this->selections)));            
        }else{
            $this->query->select($this->table_name.'.*');
        }

        if(str_contains($by, '_relation')){
            $by = $this->format_selection($by);
        }else
            $by = $this->table_name.'.'.$by;
        
        if(isset($filter_key) && isset($value)){            
            $keys = explode(',',$filter_key);
            $f = true;
            foreach($keys as $column){
                if($column=='image' || $column=='status' || str_contains($by, '_id'))
                    continue;
                
                if(str_contains($column, '_relation')){
                    $column = $this->format_selection($column);
                }else{
                    $column = $this->table_name.'.'.$column;
                }

                if($f)
                    $this->query->where($column, 'like', '%'.$value.'%');
                else    
                    $this->query->orWhere($column, 'like', '%'.$value.'%');
                $f= false;
            }
            //var_dump($this->group_by);
            if (count($this->group_by) > 0){
                //var_dump($this->group_by);
                $this->query->groupBy($this->group_by);
                $size = $this->query->groupBy($this->group_by)->get()->count();
            } else {
                $size = $this->query->count();
            }
//return $size;
        }else{
            $this->query->groupBy($this->table_name.'.id');
        }
        $this->query->offset($offset)->limit($limit);
        $this->query->orderBy($by, $order);

        //return $this->query->toSql();

        $results = $this->query->get()->toArray();

        $results = array_map(function($item){
            if(array_key_exists('status', (array)$item)){
                $item->status = ($item->status == 1) ? 1 : 0;
            };
            if(array_key_exists('avant_premiere', (array)$item)){
                $item->avant_premiere = ($item->avant_premiere == 1) ? 1 : 0;
            };
            return $item;
        },$results);
       
        if($offset == 0){            
            return array('items' => $results, 'fields' => $selections, 'size' => $size);
        }else{
            return array('items' => $results);
        }

    }

    private function join($joins, $table_parent){
        $select = '_relation';
        foreach($joins as $j => $join){
            $select .= '_'.$join->table;
            if  (isset($join->leftjoin) && $join->leftjoin == 1)
                $this->query->leftJoin($join->table, $join->table.'.'.$join->on , '=', $table_parent.'.'.$join->key);
            else
                $this->query->join($join->table, $join->table.'.'.$join->on , '=', $table_parent.'.'.$join->key);

            if (isset($join->groupby)){
                //var_dump(explode(",", $join->groupby));
                $this->group_by = array_merge(explode(",", $join->groupby), $this->group_by);
                //var_dump($this->group_by);
            }

            if(isset($join->join))
                $this->join($join->join,$join->table);
            if(isset($join->select)){
                $s = ($this->is_form ? 'id' : $join->select);
                $select .= '_'. $s;
                if (isset($join->isGroupConcat) && $join->isGroupConcat == 1)
                    array_push( $this->selections ,'GROUP_CONCAT('.$join->table.'.'.$s.') as '.$select );
                else
                    array_push( $this->selections ,'CONCAT('.$join->table.'.'.$s.') as '.$select );
                $select = '_relation';
            }
        }
    }

    private function format_selection($val){
        $strs = explode('_',$val);
        $val = $strs[count($strs)-1];
        unset($strs[count($strs)-1]);
        unset($strs[0]);
        unset($strs[1]);

        $val = implode('_',$strs).'.'.$val;
        return $val;
    }

    public function selection($table_name){
        $this->table_name = 't_'.$table_name;
        $orderby = array_reduce($this->getStruct(), function($val, $item){
            if($val)
                return $val;
            else if($item->enabled == 2 && $item->required == 1 && $item->type == 'varchar') 
                $val = $item->key;
            return $val;
        });
        $items = DB::table($this->table_name)->orderBy($orderby, 'ASC')->get();

        return $items;
    }

    public function selection2($table_name, $value){
        $this->table_name = 't_'.$table_name;
        $orderby = array_reduce($this->getStruct(), function($val, $item){
            if($val)
                return $val;
            else if($item->enabled == 2 && $item->required == 1 && $item->type == 'varchar') 
                $val = $item->key;
            return $val;
        });
        $items = DB::table($this->table_name)->where($orderby, 'like', '%'.$value.'%')->orderBy($orderby, 'ASC')->limit(10)->get();

        return $items;
    }

    public function byId($table_name, $id, Request $request){
        $this->is_form = true;
        $this->table_name = 't_'.$table_name;
        $this->query = DB::table($this->table_name);

        $joins = json_decode($request->input('joins'));
        if(isset($joins)){
            $this->join($joins, $this->table_name);
            $this->query->select($this->table_name.'.*', DB::raw(implode(',', $this->selections)));            
        }else{
            $this->query->select($this->table_name.'.*');
        }
        
        $this->query->where('t_'.$table_name.'.id', $id);
        $item = (array)$this->query->first();

        $keys= array_keys($item);
        foreach($keys as $key){
            if(preg_match('/^_relation/',$key)){
                $item[$key] =  explode(',',$item[$key]);                     
            }
        }

        if(isset($item['status'])){
            $item['status'] = ($item['status'] == 1) ? 1 : 0;
        };
        
        return $item;
    }

    public function add($table_name,Request $request){
        $this->table_name = 't_'.$table_name;
        $data = json_decode($request->input('data'), true);
        $slug = $request->input('slug'); 
        $extras = $request->input('extra');
        
        $shared = new SharedController();
        if(isset($slug)){
            $data['slug'] = $shared->formatToSlug($data[$slug]);
        }

        $data = (array)$data;
        $keys= array_keys($data);
        $id = 0;

        $update = array();
        foreach($keys as $key){
            if(preg_match('/^_relation/',$key)){
                $id_update = $data[$key];                
                unset($data[$key]); 
                
                if(!isset($extras[$key]))
                    continue;
                if ($id == 0)
                    $id = DB::table($this->table_name)->insertGetId($data); 
                $options = json_decode($extras[$key]);
                DB::table($options[0])->where($options[1], $id)->delete();
                $update[$options[1]] = $id;
                if(is_array($id_update)){
                    foreach($id_update as $sid){
                        $update[$options[2]] = $sid;
                        DB::table($options[0])->insert($update);
                    }
                }else{
                    $update[$options[2]] = $id_update;
                    DB::table($options[0])->insert($update);
                }
            }
        }

        if ($id == 0){
            $id = DB::table($this->table_name)->insertGetId($data); 
        }

        return $id;
    }

    public function update($table_name,Request $request){
        $this->table_name = 't_'.$table_name;
        $data = json_decode($request->input('data'), true);
        $slug = $request->input('slug'); 
        $extras = $request->input('extra');
        $shared = new SharedController();
        if(isset($slug)){
            $data['slug'] = $shared->formatToSlug($data[$slug]);
        }

        $id = $data['id'];
        unset($data['id']);
        $data = (array)$data;
        $keys= array_keys($data);

        $update = array();
        foreach($keys as $key){
            if(preg_match('/^_relation/',$key)){
                $id_update = $data[$key];                
                unset($data[$key]);  
                if(!isset($extras[$key]))
                    continue;
                $options = json_decode($extras[$key]);
                DB::table($options[0])->where($options[1], $id)->delete();
                $update[$options[1]] = $id;
                if(is_array($id_update)){
                    foreach($id_update as $sid){
                        $update[$options[2]] = $sid;
                        DB::table($options[0])->insert($update);
                    }
                }else{
                    $update[$options[2]] = $id_update;
                    DB::table($options[0])->insert($update);
                }
            }
        }

        return  DB::table($this->table_name)->where('id', $id)->update($data);
    }

    public function remove($table_name,Request $request){
        $this->table_name = 't_'.$table_name;
        $id = $request->input('id');
        return  DB::table($this->table_name)->where('id', $id)->delete();
    }

    public function script(){
        $result = DB::select(DB::raw('SELECT DISTINCT(titre), id, duree FROM t_cine_film ORDER BY id ASC'));
        $last = 1;
        $delete = array();
        for($i=0; $i<count($result); $i++){
            if($result[$i]->duree){
                $result[$i]->duree = str_replace("h",":",$result[$i]->duree);
                DB::table('t_cine_film')->where('id', $result[$i]->id)->update(array('duree' => $result[$i]->duree));
            }else
                DB::table('t_cine_film')->where('id', $result[$i]->id)->update(array('duree' => NULL));
        }
           
        return $result;
    }

}